/**
 *******************************************************************************
 * @file    main.c
 *
 * @brief   Main program body.
 *
 * @author  DPM
 *
 * @version V1.0.0
 *
 * @date    2024-5-6
 *
 *******************************************************************************/

/*
    Copyright (c) 2024, DP Microelectronics Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

/* Files include */
#include "main.h"
#include "dpm32m0xx.h"
#include "dpm32m0xx_eval.h"
#include "systick.h"
#include <stdio.h>

/**
  * @addtogroup DPM32M08X_LibSamples
  * @{
  */

/**
  * @addtogroup SPI
  * @{
  */

/**
  * @addtogroup SPI_MasterBlock
  * @{
  */

/* Private typedef --------------------------------------------------------------*/
GPIO_InitTypeStruct GPIO_InitStruct;
SPI_InitTypeStruct SPI_InitStruct;

/* Private define ---------------------------------------------------------------*/

/* Private macro ----------------------------------------------------------------*/

/* Private variables ------------------------------------------------------------*/
uint16_t spi_tx_buf[5] = {0x11, 0x22, 0x44, 0x88, 0xAA};
uint16_t spi_rx_buf[5] ;
char rxXferCnt = 5;
char txXferCnt = 5;

/* Private functions ------------------------------------------------------------*/

/**
  * @brief  The program entry function.
  * @retval int
  */
int main(void)
{
  /* Configure the system clock */
  SystemClockInit();
  /* Configure the system tick */
  SystickInit();
  
  /* LED Init */
  EVAL_LedInit(LED1);

  /* UART0 Init */
  EVAL_ComInit();

  /********************* Operation  *********************/
  /* Enable clock */
  RCC_Unlock();
  RCC_AHBPeriphClockCmd(RCC_AHB_PERIPH_GPIO, ENABLE);
  RCC_APBPeriphClockCmd(RCC_APB_PERIPH_SPI0, ENABLE);
  RCC_Lock();
  /* Config GPIO */
  GPIO_InitStruct.GPIO_Mode = GPIO_MODE_ALT_FUNC;
  GPIO_InitStruct.GPIO_Speed = GPIO_SPEED_HIGH;
  GPIO_InitStruct.GPIO_OType = GPIO_OTYPE_PP;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PUPD_NOPULL;
  //SCLK
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_3;
  GPIO_Init(GPIOB, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOB, GPIO_PIN_3, GPIO_ALT_FUNC_0);
  //MISO
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_4;
  GPIO_Init(GPIOB, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOB, GPIO_PIN_4, GPIO_ALT_FUNC_0);
  //MOSI
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_5;
  GPIO_Init(GPIOB, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOB, GPIO_PIN_5, GPIO_ALT_FUNC_0);
  //CS
  GPIO_InitStruct.GPIO_Mode = GPIO_MODE_OUTPUT;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PUPD_UP;
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_15;
  GPIO_Init(GPIOA, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOA, GPIO_PIN_15, GPIO_ALT_FUNC_0);
  /* Set CS */
  GPIO_SetBit(GPIOA, GPIO_PIN_15);
  /* Config SPI0 */
  SPI_InitStruct.SPI_Mode = SPI_MODE_MASTER;
  SPI_InitStruct.SPI_DataSize = SPI_DATASIZE_8BIT;
  SPI_InitStruct.SPI_CPOL = SPI_CPOL_LOW;
  SPI_InitStruct.SPI_CPHA = SPI_CPHA_1EDGE;
  SPI_InitStruct.SPI_BaudRatePrescaler = SPI_BAUDRATE_PRESCALER_128;
  SPI_InitStruct.SPI_FirstBit = SPI_FRIST_BIT_MSB;
  SPI_InitStruct.SPI_NSSSlectEn = SPI_NSS_SELECT_DISABLE;
  SPI_Init(SPI0, &SPI_InitStruct);
  /* Enable SPI0) */
  SPI_Cmd(SPI0, ENABLE);
  /* Reset CS */
  GPIO_ResetBit(GPIOA, GPIO_PIN_15);

  uint32_t time = GetTick() + 1000;
  /* SPI0 sends and receives data */
  while(rxXferCnt > 0 || txXferCnt > 0)
  {
    if(SPI_GetIntFlagStatus(SPI0, SPI_INT_FLAG_RXNE) == SET  && (rxXferCnt > 0))
    {
      spi_rx_buf[5 - rxXferCnt] = SPI_ReceiveData(SPI0);
      rxXferCnt--;
    }

    if(SPI_GetIntFlagStatus(SPI0, SPI_INT_FLAG_TXE) == SET && (txXferCnt > 0))
    {
      SPI_SendData(SPI0, spi_tx_buf[5 - txXferCnt]);
      txXferCnt--;
    }
    
    if(time < GetTick())
      break;
  }

  /* Set CS */
  GPIO_SetBit(GPIOA, GPIO_PIN_15);

  printf("\r\n********************* Trasmit  *********************\r\n");

  for(uint8_t i = 0; i < 5; i++)
  {
    printf("trasmit[%d] = 0x%x\r\n", i, spi_tx_buf[i]);
  }

  printf("\r\n********************* Receive  *********************\r\n");

  for(uint8_t i = 0; i < 5; i++)
  {
    printf("receive[%d] = 0x%x\r\n", i, spi_rx_buf[i]);
  }
  
  uint32_t ledTime = 0;
  /* Infinite loop */
  while(1)
  {
    /* Runing led toggle */
    if(ledTime < GetTick())
    {
      ledTime = GetTick() + 1000;
      EVAL_LedToggle(LED1);
    }
  }
}

/* retarget the C library printf function to the usart */
int fputc(int ch, FILE *f)
{
  UART_SendData(EVAL_COM0, (uint8_t)ch);

  while(RESET == UART_GetIntFlagStatus(EVAL_COM0, UART_INT_FLAG_TXE));

  return ch;
}



/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT DP *****END OF FILE ***************************/
